@@ -29,7 +29,7 @@ from astroid import (
     util,
 )
 from astroid.bases import BoundMethod, Generator, Instance, UnboundMethod
-from astroid.const import WIN32
+from astroid.const import PY312_PLUS, WIN32
 from astroid.exceptions import (
     AstroidBuildingError,
     AttributeInferenceError,
@@ -982,6 +982,46 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         with pytest.raises(AttributeInferenceError):
             func.getattr("")
 
+    @staticmethod
+    def test_blockstart_tolineno() -> None:
+        code = textwrap.dedent(
+            """\
+        def f1(bar: str) -> None:  #@
+            pass
+
+        def f2(  #@
+               bar: str) -> None:
+            pass
+
+        def f3(  #@
+            bar: str
+        ) -> None:
+            pass
+
+        def f4(  #@
+            bar: str
+        ):
+            pass
+
+        def f5(  #@
+               bar: str):
+            pass
+        """
+        )
+        ast_nodes: list[nodes.FunctionDef] = builder.extract_node(code)  # type: ignore[assignment]
+        assert len(ast_nodes) == 5
+
+        assert ast_nodes[0].blockstart_tolineno == 1
+
+        assert ast_nodes[1].blockstart_tolineno == 5
+
+        assert ast_nodes[2].blockstart_tolineno == 10
+
+        # Unimplemented, will return line 14 for now.
+        # assert ast_nodes[3].blockstart_tolineno == 15
+
+        assert ast_nodes[4].blockstart_tolineno == 19
+
 
 class ClassNodeTest(ModuleLoader, unittest.TestCase):
     def test_dict_interface(self) -> None:
@@ -1927,6 +1967,34 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             cls, [".E", ".C", ".A", ".B", "typing.Generic", ".D", "builtins.object"]
         )
 
+    @pytest.mark.skipif(not PY312_PLUS, reason="PEP 695 syntax requires Python 3.12")
+    def test_mro_generic_8(self):
+        cls = builder.extract_node(
+            """
+        class A: ...
+        class B[T]: ...
+        class C[T](A, B[T]): ...
+        """
+        )
+        assert isinstance(cls, nodes.ClassDef)
+        self.assertEqualMroQName(cls, [".C", ".A", ".B", "builtins.object"])
+
+    @pytest.mark.skipif(not PY312_PLUS, reason="PEP 695 syntax requires Python 3.12")
+    def test_mro_generic_9(self):
+        cls = builder.extract_node(
+            """
+        from dataclasses import dataclass
+        @dataclass
+        class A: ...
+        @dataclass
+        class B[T]: ...
+        @dataclass
+        class C[T](A, B[T]): ...
+        """
+        )
+        assert isinstance(cls, nodes.ClassDef)
+        self.assertEqualMroQName(cls, [".C", ".A", ".B", "builtins.object"])
+
     def test_mro_generic_error_1(self):
         cls = builder.extract_node(
             """
